#include <Arduino.h>

#include "common.h"
#include "adc.h"
#include <driver/temp_sensor.h>

float CalculateNtcResistance(float v_out, float v_in);

void AdcInit(void){
  analogSetAttenuation(ADC_0db); // 0 ... 0.8V range

  adcAttachPin(NTC1);
  adcAttachPin(NTC2);

  adcAttachPin(AN1);
  adcAttachPin(AN2);
  adcAttachPin(AN3);
  adcAttachPin(AN4);

  adcAttachPin(A6); // V_pv
  adcAttachPin(A7); // V_battery
  adcAttachPin(A8); // V_3V3

  temp_sensor_config_t chip_temp_sensor_config = TSENS_CONFIG_DEFAULT();
  temp_sensor_set_config(chip_temp_sensor_config);
  temp_sensor_start();
  temp_sensor_read_celsius(&T_chip);
}

float analogReadVolts(uint8_t pin){
  return analogReadMilliVolts(pin)/1e3;
}

void AdcReadValues(void) {
  V_pv = analogReadVolts(A6)*ADC_VOLTAGE_CAL;
  V_battery = analogReadVolts(A7)*ADC_VOLTAGE_CAL;
  V_3V3 = analogReadVolts(A8)*ADC_VOLTAGE_CAL;

  R_NTC1 = CalculateNtcResistance(analogReadVolts(NTC1),V_3V3);
  R_NTC2 = CalculateNtcResistance(analogReadVolts(NTC2),V_3V3);

  V_AN1 = analogReadVolts(AN1);
  V_AN2 = analogReadVolts(AN2);
  V_AN3 = analogReadVolts(AN3);
  V_AN4 = analogReadVolts(AN4);

  temp_sensor_read_celsius(&T_chip);
}

float CalculateNtcResistance(float v_out, float v_in){
  return NTC_R_PULLUP * (v_out/(v_in - v_out));
}
