/*
 * SDR Engineering / NanoPower Dev S2 demo code
 * copyright SDR Engineering (c)
 *
 * written by Bas Vermulst, 2024
 *
 */

#include "common.h"
#include "adc.h"
#include "wifi_client.h"

#include <ArduinoOTA.h> // for OTA

void setup() {
  timeStart=millis();
  wakeups++; // we woke up again, hello world :-)

  Serial.begin(115200);
  DebugPrintln("Starting...");
  
  gpio_hold_dis((gpio_num_t) EN_5V_PSU_PIN);
  gpio_hold_dis((gpio_num_t) EN_PWR1_PIN);
  gpio_hold_dis((gpio_num_t) EN_PWR2_PIN);
  gpio_deep_sleep_hold_dis(); // disable the deep-sleep pin hold function

  pinMode(LED1, OUTPUT);
  pinMode(EN_5V_PSU_PIN, OUTPUT);
  pinMode(EN_PWR1_PIN, OUTPUT);
  pinMode(EN_PWR2_PIN, OUTPUT);

  pinMode(GPIO1, INPUT); // example where GPIO1 is used as input, rest is output
  pinMode(GPIO2, OUTPUT);
  pinMode(GPIO3, OUTPUT);
  pinMode(GPIO4, OUTPUT);

  digitalWrite(EN_5V_PSU_PIN, 0); // set 5V0 power supply to disabled
  digitalWrite(EN_PWR1_PIN, 0); // disable 5V0 supply output 1
  digitalWrite(EN_PWR2_PIN, 0); // disable 5V0 power supply output 2

  digitalWrite(LED1, 1); // enable LED, be aware this consumes 5 mA

  AdcInit();  
  timeAdc=millis();
  DebugPrintln("Time to adc=" + timeAdc);

  WiFiInitSta();
  timeConnectWiFi=millis();
  DebugPrintln("Time to wifi=" + timeConnectWiFi);
  
  ArduinoOTA.begin();
}






void loop() {
  // Note: this is a bad example!
  // For minimum energy consumption, do not use the main loop and avoid using delays. Just do what you have to do, and go back to sleep.
  // Here, the main loop is just used to show how to use the various features of the board. It's bad practice.
  // If you want to keep data in RAM while the ESP is in deep sleep, use variables declared as RTC_DATA_ATTR, e.g. "RTC_DATA_ATTR int keep_while_sleeping;"

  static int state=0;
  
  ArduinoOTA.handle(); // keep this in the loop for OTA update functionality

  AdcReadValues(); // get values from analog input, this fills the various ADC variables with their respective values
  // NTC values are read back as resistor value. Implement a LUT to find the temperature.

  DebugPrintln("");
  DebugPrintln("");
  DebugPrintln("ADC status");
  DebugPrintln("-----------");
  DebugPrintln("R_NTC1=" + String(R_NTC1,1));
  DebugPrintln("R_NTC2=" + String(R_NTC2,1));
  DebugPrintln("AN1=" + String(V_AN1,3));
  DebugPrintln("AN2=" + String(V_AN2,3));
  DebugPrintln("AN3=" + String(V_AN3,3));
  DebugPrintln("AN4=" + String(V_AN4,3));
  DebugPrintln("V_pv=" + String(V_pv,3));
  DebugPrintln("V_battery=" + String(V_battery,3));
  DebugPrintln("V_3V3=" + String(V_3V3,3));
  DebugPrintln("");
  DebugPrintln("GPIO status");
  DebugPrintln("-----------");
  DebugPrintln("GPIO1=" + String(digitalRead(GPIO1)));
  DebugPrintln("GPIO2=" + String(digitalRead(GPIO2)));
  DebugPrintln("GPIO3=" + String(digitalRead(GPIO3)));
  DebugPrintln("GPIO4=" + String(digitalRead(GPIO4)));
  DebugPrintln("");
  DebugPrintln("PWR status");
  DebugPrintln("-----------");
  DebugPrintln("PWR1=" + String(digitalRead(EN_PWR1_PIN)));
  DebugPrintln("PWR2=" + String(digitalRead(EN_PWR2_PIN)));
  DebugPrintln("PSU=" + String(digitalRead(EN_5V_PSU_PIN)));

  if(millis() > 30e3 && state == 0){
    // after 30 seconds, enable 5V0 supply and PWR1 output, just for fun
    // note: to get voltage on the PWRx pins, you need to activate the 5V0 power supply
    digitalWrite(EN_5V_PSU_PIN, true);
    digitalWrite(EN_PWR1_PIN, true);

    state++;
  }

  if(millis() > 60e3){
    // after 60 seconds, go to sleep
    // if connected to anything, now is the time to disconnect (e.g. mqtt.disconnect();)

    WiFi.disconnect(); // graceful disconnect from wifi
    WiFi.mode(WIFI_OFF); // radio must be turned off before going to deep sleep

    digitalWrite(EN_PWR1_PIN, 0); // keep PWR1 disabled during sleep
    digitalWrite(EN_PWR2_PIN, 1); // keep PWR2 enabled during sleep
    digitalWrite(EN_5V_PSU_PIN, 1); // since we have one power output enabled, we need to keep the 5V0 supply active while sleeping. be aware this consumes current!
    digitalWrite(LED1, 0);

    gpio_hold_en((gpio_num_t) EN_5V_PSU_PIN);
    gpio_hold_en((gpio_num_t) EN_PWR1_PIN);
    gpio_hold_en((gpio_num_t) EN_PWR2_PIN);
    gpio_deep_sleep_hold_en();

    delay(1);

    esp_sleep_enable_timer_wakeup(WAKE_SLEEP_INTERVAL*1e6); // standby period is in s, function accepts us
    esp_deep_sleep_start(); // sweet dreams
  }

  delay(5000);
}
